@@ -25,26 +25,29 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.
 """
 
-__version__ = '1.7.1'
+__version__ = '2.0.0'
 
 import asyncio
 from sys import platform, version_info
 
 from .client import Client, run_multiple, start_multiple, close_multiple
-from .auth import (EmailAndPasswordAuth, ExchangeCodeAuth, DeviceAuth,
-                   RefreshTokenAuth, AdvancedAuth)
+from .auth import (Auth, EmailAndPasswordAuth, ExchangeCodeAuth,
+                   DeviceAuth, RefreshTokenAuth, AdvancedAuth)
 from .friend import Friend, PendingFriend
 from .message import FriendMessage, PartyMessage
-from .party import (PartyMember, ClientPartyMember, Party, ClientParty,
-                    PartyInvitation, PartyJoinConfirmation)
+from .party import (DefaultPartyConfig, DefaultPartyMemberConfig, PartyMember,
+                    ClientPartyMember, JustChattingClientPartyMember, Party, 
+                    ClientParty, ReceivedPartyInvitation, PartyJoinConfirmation)
 from .presence import Presence, PresenceGameplayStats, PresenceParty
-from .user import ClientUser, User, BlockedUser, ExternalAuth
+from .user import (ClientUser, User, BlockedUser, ExternalAuth,
+                   ProfileSearchEntryUser, SacSearchEntryUser)
 from .stats import StatsV2
 from .enums import *
 from .errors import *
 from .store import Store, FeaturedStoreItem, DailyStoreItem
 from .news import BattleRoyaleNewsPost
 from .playlist import Playlist
+from .kairos import Avatar
 
 # fix for python 3.8
 if platform == 'win32' and version_info >= (3, 8):
